@@ -108,6 +108,10 @@ typedef enum
 #define isutf8_3_2(c) \
 	(((c) & 0xc0) == 0x80)
 
+#define isutf8bom(s) \
+	(((*(s)) & 0xff) == 0xef && ((*(s + 1)) & 0xff) == 0xbb && \
+	 ((*(s + 2)) & 0xff) == 0xbf)
+
 #define K_IN()				\
 	if (state != JIS_KANJI) {	\
 		*out++ = ESC;		\
@@ -831,6 +835,12 @@ static gchar *conv_anytoutf8(const gchar *inbuf, gint *error)
 		return conv_sjistoutf8(inbuf, error);
 	case C_EUC_JP:
 		return conv_euctoutf8(inbuf, error);
+	case C_UTF_8:
+		if (error)
+			*error = 0;
+		if (isutf8bom(inbuf))
+			inbuf += 3;
+		return g_strdup(inbuf);
 	default:
 		if (error)
 			*error = 0;
@@ -864,6 +874,8 @@ static gchar *conv_utf8tosjis(const gchar *inbuf, gint *error)
 		}
 	}
 
+	if (isutf8bom(inbuf))
+		inbuf += 3;
 	return conv_iconv_strdup_with_cd(inbuf, cd, error);
 }
 
@@ -893,6 +905,8 @@ static gchar *conv_utf8toeuc(const gchar *inbuf, gint *error)
 		}
 	}
 
+	if (isutf8bom(inbuf))
+		inbuf += 3;
 	return conv_iconv_strdup_with_cd(inbuf, cd, error);
 }
 
@@ -1217,6 +1231,8 @@ gchar *conv_utf8todisp(const gchar *inbuf, gint *error)
 	if (g_utf8_validate(inbuf, -1, NULL) == TRUE) {
 		if (error)
 			*error = 0;
+		if (isutf8bom(inbuf))
+			inbuf += 3;
 		return g_strdup(inbuf);
 	} else
 		return conv_ustodisp(inbuf, error);
@@ -1312,7 +1328,9 @@ void conv_code_converter_destroy(CodeConverter *conv)
 
 gchar *conv_convert(CodeConverter *conv, const gchar *inbuf)
 {
-	if (conv->code_conv_func != conv_noconv)
+	if (!inbuf)
+		return NULL;
+	else if (conv->code_conv_func != conv_noconv)
 		return conv->code_conv_func(inbuf, NULL);
 	else
 		return conv_iconv_strdup
@@ -1326,6 +1344,12 @@ gchar *conv_codeset_strdup_full(const gchar *inbuf,
 {
 	CodeConvFunc conv_func;
 
+	if (!inbuf) {
+		if (error)
+			*error = 0;
+		return NULL;
+	}
+
 	src_encoding = conv_get_fallback_for_private_encoding(src_encoding);
 
 	conv_func = conv_get_code_conv_func(src_encoding, dest_encoding);
@@ -1384,12 +1408,20 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_encoding,
 			code_conv = conv_jistodisp;
 		else if (dest_charset == C_EUC_JP)
 			code_conv = conv_jistoeuc;
+		else if (dest_charset == C_SHIFT_JIS ||
+			 dest_charset == C_CP932)
+			code_conv = conv_jistosjis;
 		else if (dest_charset == C_UTF_8)
 			code_conv = conv_jistoutf8;
 		break;
 	case C_SHIFT_JIS:
+	case C_CP932:
 		if (dest_charset == C_AUTO)
 			code_conv = conv_sjistodisp;
+		else if (dest_charset == C_ISO_2022_JP   ||
+			 dest_charset == C_ISO_2022_JP_2 ||
+			 dest_charset == C_ISO_2022_JP_3)
+			code_conv = conv_sjistojis;
 		else if (dest_charset == C_EUC_JP)
 			code_conv = conv_sjistoeuc;
 		else if (dest_charset == C_UTF_8)
@@ -1412,6 +1444,9 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_encoding,
 			 dest_charset == C_ISO_2022_JP_2 ||
 			 dest_charset == C_ISO_2022_JP_3)
 			code_conv = conv_utf8tojis;
+		else if (dest_charset == C_SHIFT_JIS ||
+			 dest_charset == C_CP932)
+			code_conv = conv_utf8tosjis;
 		break;
 	default:
 		break;
@@ -1459,6 +1494,12 @@ gchar *conv_iconv_strdup_with_cd(const gchar *inbuf, iconv_t cd, gint *error)
 	size_t len;
 	gint error_ = 0;
 
+	if (!inbuf) {
+		if (error)
+			*error = 0;
+		return NULL;
+	}
+
 	inbuf_p = inbuf;
 	in_size = strlen(inbuf);
 	in_left = in_size;
@@ -2134,6 +2175,8 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 	left = MAX_LINELEN - header_len;
 
 	while (*srcp) {
+		gboolean in_quote = FALSE;
+
 		LBREAK_IF_REQUIRED(left <= 0, TRUE);
 
 		while (g_ascii_isspace(*srcp)) {
@@ -2178,13 +2221,18 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 			gboolean cont = FALSE;
 
 			while (*p != '\0') {
-				if (g_ascii_isspace(*p) &&
-				    !is_next_nonascii(p + 1))
-					break;
-				/* don't include parentheses in encoded
-				   strings */
-				if (addr_field && (*p == '(' || *p == ')'))
-					break;
+				if (*p == '"')
+					in_quote ^= TRUE;
+				else if (!in_quote) {
+					if (g_ascii_isspace(*p) &&
+					    !is_next_nonascii(p + 1))
+						break;
+					/* don't include parentheses in encoded
+					   strings */
+					if (addr_field &&
+					    (*p == '(' || *p == ')'))
+						break;
+				}
 
 				mb_len = g_utf8_skip[*(guchar *)p];
 
@@ -2467,6 +2515,56 @@ gint conv_copy_dir(const gchar *src, const gchar *dest, const gchar *encoding)
 	return 0;
 }
 
+CharSet conv_check_file_encoding(const gchar *file)
+{
+	FILE *fp;
+	gchar buf[BUFFSIZE];
+	CharSet enc;
+	const gchar *enc_str;
+	gboolean is_locale = TRUE, is_utf8 = TRUE;
+
+	g_return_val_if_fail(file != NULL, C_AUTO);
+
+	enc = conv_get_locale_charset();
+	enc_str = conv_get_locale_charset_str();
+	if (enc == C_UTF_8)
+		is_locale = FALSE;
+
+	if ((fp = g_fopen(file, "rb")) == NULL) {
+		FILE_OP_ERROR(file, "fopen");
+		return C_AUTO;
+	}
+
+	while (fgets(buf, sizeof(buf), fp) != NULL) {
+		gchar *str;
+		gint error = 0;
+
+		if (is_locale) {
+			str = conv_codeset_strdup_full(buf, enc_str,
+						       CS_INTERNAL, &error);
+			if (!str || error != 0)
+				is_locale = FALSE;
+			g_free(str);
+		}
+
+		if (is_utf8 && g_utf8_validate(buf, -1, NULL) == FALSE) {
+			is_utf8 = FALSE;
+		}
+
+		if (!is_locale && !is_utf8)
+			break;
+	}
+
+	fclose(fp);
+
+	if (is_locale)
+		return enc;
+	else if (is_utf8)
+		return C_UTF_8;
+	else
+		return C_AUTO;
+}
+
 gchar *conv_filename_from_utf8(const gchar *utf8_file)
 {
 	gchar *fs_file;
