@@ -83,6 +83,8 @@ static PrefParam param[] = {
 	 &prefs_common.reply_account_autosel, P_BOOL},
 	{"default_reply_list", "TRUE", &prefs_common.default_reply_list,
 	 P_BOOL},
+	{"inherit_recipient_on_self_reply", "FALSE",
+	 &prefs_common.inherit_recipient_on_self_reply, P_BOOL},
 
 	{"show_ruler", "TRUE", &prefs_common.show_ruler, P_BOOL},
 
@@ -320,9 +322,13 @@ static PrefParam param[] = {
 	 &prefs_common.always_show_msg, P_BOOL},
 	{"open_unread_on_enter", "FALSE", &prefs_common.open_unread_on_enter,
 	 P_BOOL},
+	{"remember_last_selected", "FALSE",
+	 &prefs_common.remember_last_selected, P_BOOL},
 	{"mark_as_read_on_new_window", "FALSE",
 	 &prefs_common.mark_as_read_on_new_window, P_BOOL},
 	{"open_inbox_on_inc", "FALSE", &prefs_common.open_inbox_on_inc, P_BOOL},
+	{"open_inbox_on_startup", "FALSE", &prefs_common.open_inbox_on_startup,
+	 P_BOOL},
 	{"immediate_execution", "TRUE", &prefs_common.immediate_exec, P_BOOL},
 
 #ifdef G_OS_WIN32
@@ -332,6 +338,7 @@ static PrefParam param[] = {
 #endif
 
 	{"show_trayicon", "TRUE", &prefs_common.show_trayicon, P_BOOL},
+	{"minimize_to_tray", "FALSE", &prefs_common.minimize_to_tray, P_BOOL},
 
 	/* Other */
 	{"receive_dialog_mode", "1", &prefs_common.recv_dialog_mode, P_ENUM},
@@ -359,11 +366,6 @@ static PrefParam param[] = {
 	{"uri_open_command", DEFAULT_BROWSER_CMD, &prefs_common.uri_cmd,
 #endif
 	 P_STRING},
-#ifdef G_OS_WIN32
-	{"print_command", NULL, &prefs_common.print_cmd, P_STRING},
-#else
-	{"print_command", "lpr %s", &prefs_common.print_cmd, P_STRING},
-#endif
 #ifdef G_OS_WIN32
 	{"ext_editor_command", "notepad '%s'", &prefs_common.ext_editor_cmd,
 #else
@@ -371,6 +373,13 @@ static PrefParam param[] = {
 #endif
 	 P_STRING},
 
+	{"use_print_command", "FALSE", &prefs_common.use_print_cmd, P_BOOL},
+#ifdef G_OS_WIN32
+	{"print_command", NULL, &prefs_common.print_cmd, P_STRING},
+#else
+	{"print_command", "lpr %s", &prefs_common.print_cmd, P_STRING},
+#endif
+
 	{"use_ext_inc", "FALSE", &prefs_common.use_extinc, P_BOOL},
 	{"ext_inc_path", DEFAULT_INC_PATH, &prefs_common.extinc_cmd, P_STRING},
 	{"use_ext_sendmail", "FALSE", &prefs_common.use_extsend, P_BOOL},
@@ -386,6 +395,11 @@ static PrefParam param[] = {
 };
 
 
+PrefsCommon *prefs_common_get(void)
+{
+	return &prefs_common;
+}
+
 PrefParam *prefs_common_get_params(void)
 {
 	return param;
@@ -462,25 +476,17 @@ void prefs_common_write_config(void)
 	g_free(path);
 }
 
-void prefs_common_junk_filter_list_set(void)
+static FilterRule *prefs_common_junk_filter_rule_create(gboolean is_manual)
 {
 	FilterRule *rule;
 	FilterCond *cond;
 	FilterAction *action;
 	GSList *cond_list = NULL, *action_list = NULL;
 
-	if (prefs_common.junk_fltlist) {
-		filter_rule_list_free(prefs_common.junk_fltlist);
-		prefs_common.junk_fltlist = NULL;
-	}
-
-	if (!prefs_common.junk_classify_cmd || !prefs_common.junk_folder)
-		return;
-
 	cond = filter_cond_new(FLT_COND_CMD_TEST, 0, 0, NULL,
 			       prefs_common.junk_classify_cmd);
 	cond_list = g_slist_append(NULL, cond);
-	if (prefs_common.delete_junk_on_recv) {
+	if (prefs_common.delete_junk_on_recv && !is_manual) {
 		action = filter_action_new(FLT_ACTION_COPY,
 					   prefs_common.junk_folder);
 		action_list = g_slist_append(NULL, action);
@@ -497,8 +503,35 @@ void prefs_common_junk_filter_list_set(void)
 		action_list = g_slist_append(action_list, action);
 	}
 
-	rule = filter_rule_new(_("Junk mail filter"), FLT_OR,
-			       cond_list, action_list);
+	if (is_manual)
+		rule = filter_rule_new(_("Junk mail filter (manual)"), FLT_OR,
+	 			       cond_list, action_list);
+	else
+		rule = filter_rule_new(_("Junk mail filter"), FLT_OR,
+				       cond_list, action_list);
 
+	return rule;
+}
+
+void prefs_common_junk_filter_list_set(void)
+{
+	FilterRule *rule;
+
+	if (prefs_common.junk_fltlist) {
+		filter_rule_list_free(prefs_common.junk_fltlist);
+		prefs_common.junk_fltlist = NULL;
+	}
+	if (prefs_common.manual_junk_fltlist) {
+		filter_rule_list_free(prefs_common.manual_junk_fltlist);
+		prefs_common.manual_junk_fltlist = NULL;
+	}
+
+	if (!prefs_common.junk_classify_cmd || !prefs_common.junk_folder)
+		return;
+
+	rule = prefs_common_junk_filter_rule_create(FALSE);
 	prefs_common.junk_fltlist = g_slist_append(NULL, rule);
+
+	rule = prefs_common_junk_filter_rule_create(TRUE);
+	prefs_common.manual_junk_fltlist = g_slist_append(NULL, rule);
 }
