@@ -68,7 +68,6 @@ void procmime_mimeinfo_free_all(MimeInfo *mimeinfo)
 		g_free(mimeinfo->content_disposition);
 		g_free(mimeinfo->filename);
 
-		g_free(mimeinfo->plaintextfile);
 		g_free(mimeinfo->sigstatus);
 		g_free(mimeinfo->sigstatus_full);
 
@@ -101,6 +100,7 @@ MimeInfo *procmime_mimeinfo_insert(MimeInfo *parent, MimeInfo *mimeinfo)
 	return mimeinfo;
 }
 
+#if 0
 void procmime_mimeinfo_replace(MimeInfo *old, MimeInfo *new)
 {
 	MimeInfo *parent = old->parent;
@@ -126,6 +126,7 @@ void procmime_mimeinfo_replace(MimeInfo *old, MimeInfo *new)
 		child = new;
 	}
 }
+#endif
 
 MimeInfo *procmime_mimeinfo_next(MimeInfo *mimeinfo)
 {
@@ -401,7 +402,7 @@ static gchar *procmime_find_parameter_delimiter(const gchar *param,
 	while (*p) {
 		if (*p == '=')
 			break;
-		else if (*p == ';' || g_ascii_isspace(*p)) {
+		else if (*p == ';' || *p == '\r' || *p == '\n') {
 			delim = p;
 			break;
 		}
@@ -414,6 +415,8 @@ static gchar *procmime_find_parameter_delimiter(const gchar *param,
 	*eq = p;
 
 	++p;
+	while (g_ascii_isspace(*p))
+		++p;
 	if (*p == '"') {
 		quoted = TRUE;
 		++p;
@@ -423,7 +426,7 @@ static gchar *procmime_find_parameter_delimiter(const gchar *param,
 		if (quoted == TRUE) {
 			if (*p == '"')
 				quoted = FALSE;
-		} else if (*p == ';' || g_ascii_isspace(*p)) {
+		} else if (*p == ';' || *p == '\r' || *p == '\n') {
 			delim = p;
 			break;
 		}
@@ -497,6 +500,7 @@ static MimeParams *procmime_parse_mime_parameter(const gchar *str)
 			param = g_strdup(p);
 
 		name = g_strndup(p, eq - p);
+		g_strchomp(name);
 		if (*name != '*' && (ast = strchr(name, '*'))) {
 			const gchar *next = ast + 1;
 
@@ -525,6 +529,7 @@ static MimeParams *procmime_parse_mime_parameter(const gchar *str)
 		}
 
 		value = g_strdup(param + (eq - p) + 1);
+		g_strstrip(value);
 		if (*value == '"')
 			extract_quote(value, '"');
 
@@ -830,6 +835,12 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 		}
 
 		if (normalize_lbreak) {
+			if (fflush(tmpfp) == EOF) {
+				perror("fflush");
+				fclose(tmpfp);
+				if (tmp_file) fclose(outfp);
+				return NULL;
+			}
 			rewind(tmpfp);
 			while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
 #ifdef G_OS_WIN32
@@ -873,6 +884,12 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 		base64_decoder_free(decoder);
 
 		if (normalize_lbreak) {
+			if (fflush(tmpfp) == EOF) {
+				perror("fflush");
+				fclose(tmpfp);
+				if (tmp_file) fclose(outfp);
+				return NULL;
+			}
 			rewind(tmpfp);
 			while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
 #ifdef G_OS_WIN32
@@ -925,6 +942,14 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 		}
 	}
 
+	if (fflush(outfp) == EOF)
+		perror("fflush");
+	if (ferror(outfp) != 0) {
+		g_warning("procmime_decode_content(): Can't write to temporary file\n");
+		if (tmp_file) fclose(outfp);
+		return NULL;
+	}
+
 	if (tmp_file) rewind(outfp);
 	return outfp;
 }
@@ -970,7 +995,11 @@ gint procmime_get_part_fp(const gchar *outfile, FILE *infp, MimeInfo *mimeinfo)
 	while (fgets(buf, sizeof(buf), infp) != NULL)
 		if (buf[0] == '\r' || buf[0] == '\n') break;
 
-	procmime_decode_content(outfp, infp, mimeinfo);
+	if (procmime_decode_content(outfp, infp, mimeinfo) == NULL) {
+		fclose(outfp);
+		g_unlink(outfile);
+		return -1;
+	}
 
 	if (fclose(outfp) == EOF) {
 		FILE_OP_ERROR(outfile, "fclose");
@@ -1100,6 +1129,11 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo, FILE *infp,
 		g_warning(_("procmime_get_text_content(): Code conversion failed.\n"));
 
 	fclose(tmpfp);
+	if (fflush(outfp) == EOF) {
+		perror("fflush");
+		fclose(outfp);
+		return NULL;
+	}
 	rewind(outfp);
 
 	return outfp;
