@@ -23,6 +23,7 @@
 #include <glib/gi18n.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <errno.h>
 
 #include "utils.h"
 #include "procmsg.h"
@@ -156,10 +157,11 @@ gint procmsg_read_cache_data_str(FILE *fp, gchar **str)
 #define READ_CACHE_DATA(data, fp)				\
 {								\
 	if (procmsg_read_cache_data_str(fp, &data) < 0) {	\
+		g_warning("Cache data is corrupted\n");		\
 		procmsg_msginfo_free(msginfo);			\
 		procmsg_msg_list_free(mlist);			\
-		mlist = NULL;					\
-		break;						\
+		fclose(fp);					\
+		return NULL;					\
 	}							\
 }
 
@@ -171,8 +173,8 @@ gint procmsg_read_cache_data_str(FILE *fp, gchar **str)
 		g_warning("Cache data is corrupted\n");		\
 		procmsg_msginfo_free(msginfo);			\
 		procmsg_msg_list_free(mlist);			\
-		mlist = NULL;					\
-		break;						\
+		fclose(fp);					\
+		return NULL;					\
 	} else							\
 		n = idata;					\
 }
@@ -380,6 +382,40 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 	g_hash_table_destroy(mark_table);
 }
 
+static void mark_all_read_func(gpointer key, gpointer value, gpointer data)
+{
+	MSG_UNSET_PERM_FLAGS(*((MsgFlags *)value), MSG_NEW|MSG_UNREAD);
+}
+
+void procmsg_mark_all_read(FolderItem *item)
+{
+	GHashTable *mark_table;
+
+	debug_print("Marking all messages as read\n");
+
+	mark_table = procmsg_read_mark_file(item);
+	if (mark_table) {
+		g_hash_table_foreach(mark_table, mark_all_read_func, NULL);
+		procmsg_write_mark_file(item, mark_table);
+		hash_free_value_mem(mark_table);
+		g_hash_table_destroy(mark_table);
+	}
+
+	if (item->mark_queue) {
+		GSList *cur;
+		MsgInfo *msginfo;
+
+		for (cur = item->mark_queue; cur != NULL; cur = cur->next) {
+			msginfo = (MsgInfo *)cur->data;
+			MSG_UNSET_PERM_FLAGS
+				(msginfo->flags, MSG_NEW|MSG_UNREAD);
+		}
+		item->mark_dirty = TRUE;
+	}
+
+	item->new = item->unread = 0;
+}
+
 static FolderSortType cmp_func_sort_type;
 
 GSList *procmsg_sort_msg_list(GSList *mlist, FolderSortKey sort_key,
@@ -754,7 +790,8 @@ static void procmsg_write_mark_file(FolderItem *item, GHashTable *mark_table)
 {
 	FILE *fp;
 
-	fp = procmsg_open_mark_file(item, DATA_WRITE);
+	if ((fp = procmsg_open_mark_file(item, DATA_WRITE)) == NULL)
+		return;
 	g_hash_table_foreach(mark_table, write_mark_func, fp);
 	fclose(fp);
 }
@@ -763,14 +800,22 @@ FILE *procmsg_open_data_file(const gchar *file, guint version,
 			     DataOpenMode mode, gchar *buf, size_t buf_size)
 {
 	FILE *fp;
-	guint32 data_ver;
+	guint32 data_ver = 0;
 
 	g_return_val_if_fail(file != NULL, NULL);
 
 	if (mode == DATA_WRITE) {
 		if ((fp = g_fopen(file, "wb")) == NULL) {
-			FILE_OP_ERROR(file, "fopen");
-			return NULL;
+			if (errno == EACCES) {
+				change_file_mode_rw(NULL, file);
+				if ((fp = g_fopen(file, "wb")) == NULL) {
+					FILE_OP_ERROR(file, "fopen");
+					return NULL;
+				}
+			} else {
+				FILE_OP_ERROR(file, "fopen");
+				return NULL;
+			}
 		}
 		if (change_file_mode_rw(fp, file) < 0)
 			FILE_OP_ERROR(file, "chmod");
@@ -780,9 +825,18 @@ FILE *procmsg_open_data_file(const gchar *file, guint version,
 	}
 
 	/* check version */
-	if ((fp = g_fopen(file, "rb")) == NULL)
-		debug_print("Mark/Cache file '%s' not found\n", file);
-	else {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
+		if (errno == EACCES) {
+			change_file_mode_rw(NULL, file);
+			if ((fp = g_fopen(file, "rb")) == NULL) {
+				FILE_OP_ERROR(file, "fopen");
+			}
+		} else {
+			debug_print("Mark/Cache file '%s' not found\n", file);
+		}
+	}
+
+	if (fp) {
 		if (buf && buf_size > 0)
 			setvbuf(fp, buf, _IOFBF, buf_size);
 		if (fread(&data_ver, sizeof(data_ver), 1, fp) != 1 ||
@@ -800,8 +854,16 @@ FILE *procmsg_open_data_file(const gchar *file, guint version,
 	if (fp) {
 		/* reopen with append mode */
 		fclose(fp);
-		if ((fp = g_fopen(file, "ab")) == NULL)
-			FILE_OP_ERROR(file, "fopen");
+		if ((fp = g_fopen(file, "ab")) == NULL) {
+			if (errno == EACCES) {
+				change_file_mode_rw(NULL, file);
+				if ((fp = g_fopen(file, "ab")) == NULL) {
+					FILE_OP_ERROR(file, "fopen");
+				}
+			} else {
+				FILE_OP_ERROR(file, "fopen");
+			}
+		}
 	} else {
 		/* open with overwrite mode if mark file doesn't exist or
 		   version is different */
@@ -1044,8 +1106,8 @@ gchar *procmsg_get_message_file_path(MsgInfo *msginfo)
 
 	g_return_val_if_fail(msginfo != NULL, NULL);
 
-	if (msginfo->plaintext_file)
-		file = g_strdup(msginfo->plaintext_file);
+	if (msginfo->encinfo && msginfo->encinfo->plaintext_file)
+		file = g_strdup(msginfo->encinfo->plaintext_file);
 	else if (msginfo->file_path)
 		return g_strdup(msginfo->file_path);
 	else {
@@ -1204,7 +1266,12 @@ gboolean procmsg_trash_messages_exist(void)
 
 void procmsg_empty_trash(FolderItem *trash)
 {
-	if (trash && trash->total > 0) {
+	if (!trash)
+		return;
+
+	g_return_if_fail(trash->stype == F_TRASH);
+
+	if (trash->total > 0) {
 		debug_print("Emptying messages in %s ...\n", trash->path);
 
 		folder_item_remove_all_msg(trash);
@@ -1280,16 +1347,52 @@ gint procmsg_save_to_outbox(FolderItem *outbox, const gchar *file)
 	return 0;
 }
 
+static guint print_id = 0;
+
+static gint print_command_exec(const gchar *file, const gchar *cmdline)
+{
+	static const gchar *def_cmd = "lpr %s";
+	gchar buf[1024];
+
+#ifdef G_OS_WIN32
+	if (canonicalize_file_replace(file) < 0)
+		return -1;
+#endif
+
+	if (cmdline && str_find_format_times(cmdline, 's') == 1)
+		g_snprintf(buf, sizeof(buf) - 1, cmdline, file);
+	else {
+		if (cmdline) {
+			g_warning(_("Print command line is invalid: `%s'\n"),
+				  cmdline);
+			return -1;
+		}
+
+#ifdef G_OS_WIN32
+		execute_print_file(file);
+		return 0;
+#else
+		g_snprintf(buf, sizeof(buf) - 1, def_cmd, file);
+#endif
+	}
+
+	g_strchomp(buf);
+	if (buf[strlen(buf) - 1] != '&')
+		strcat(buf, "&");
+
+	system(buf);
+
+	return 0;
+}
+
 void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline,
 			   gboolean all_headers)
 {
-	static const gchar *def_cmd = "lpr %s";
-	static guint id = 0;
 	gchar *prtmp;
 	FILE *msgfp, *tmpfp, *prfp;
 	GPtrArray *headers;
 	gint i;
-	gchar buf[1024];
+	gchar buf[BUFFSIZE];
 
 	g_return_if_fail(msginfo != NULL);
 
@@ -1300,7 +1403,8 @@ void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline,
 	}
 
 	prtmp = g_strdup_printf("%s%cprinttmp-%08x.txt",
-				get_mime_tmp_dir(), G_DIR_SEPARATOR, id++);
+				get_mime_tmp_dir(), G_DIR_SEPARATOR,
+				print_id++);
 
 	if ((prfp = g_fopen(prtmp, "wb")) == NULL) {
 		FILE_OP_ERROR(prtmp, "fopen");
@@ -1347,11 +1451,16 @@ void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline,
 				body++;
 		}
 
-		locale_str = conv_codeset_strdup
-			(body, CS_INTERNAL, conv_get_locale_charset_str());
-		fprintf(prfp, "%s: %s\n", hdr->name,
-			locale_str ? locale_str : body);
-		g_free(locale_str);
+		if (body && *body != '\0') {
+			locale_str = conv_codeset_strdup
+				(body, CS_INTERNAL,
+				 conv_get_locale_charset_str());
+			fprintf(prfp, "%s: %s\n", hdr->name,
+				locale_str ? locale_str : body);
+			g_free(locale_str);
+		} else {
+			fprintf(prfp, "%s: (none)\n", hdr->name);
+		}
 	}
 
 	procheader_header_array_destroy(headers);
@@ -1364,37 +1473,48 @@ void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline,
 	fclose(prfp);
 	fclose(tmpfp);
 
-#ifdef G_OS_WIN32
-	if (canonicalize_file_replace(prtmp) < 0) {
-		g_free(prtmp);
-		return;
-	}
-#endif
-
-	if (cmdline && str_find_format_times(cmdline, 's') == 1)
-		g_snprintf(buf, sizeof(buf) - 1, cmdline, prtmp);
-	else {
-		if (cmdline) {
-			g_warning(_("Print command line is invalid: `%s'\n"),
-				  cmdline);
-			g_free(prtmp);
-			return;
-		}
-
-#ifdef G_OS_WIN32
-		execute_print_file(prtmp);
-		g_free(prtmp);
-		return;
-#else
-		g_snprintf(buf, sizeof(buf) - 1, def_cmd, prtmp);
-#endif
-	}
+	print_command_exec(prtmp, cmdline);
 
 	g_free(prtmp);
+}
 
-	g_strchomp(buf);
-	if (buf[strlen(buf) - 1] != '&') strcat(buf, "&");
-	system(buf);
+void procmsg_print_message_part(MsgInfo *msginfo, MimeInfo *partinfo,
+				const gchar *cmdline, gboolean all_headers)
+{
+	FILE *msgfp, *tmpfp, *prfp;
+	gchar *prtmp;
+	gchar buf[BUFFSIZE];
+
+	if ((msgfp = procmsg_open_message(msginfo)) == NULL) {
+		return;
+	}
+
+	if ((tmpfp = procmime_get_text_content
+		(partinfo, msgfp, conv_get_locale_charset_str())) == NULL) {
+		fclose(msgfp);
+		return;
+	}
+	fclose(msgfp);
+
+	prtmp = g_strdup_printf("%s%cprinttmp-%08x.txt",
+				get_mime_tmp_dir(), G_DIR_SEPARATOR,
+				print_id++);
+	if ((prfp = g_fopen(prtmp, "wb")) == NULL) {
+		FILE_OP_ERROR(prtmp, "fopen");
+		g_free(prtmp);
+		fclose(tmpfp);
+		return;
+	}
+
+	while (fgets(buf, sizeof(buf), tmpfp) != NULL)
+		fputs(buf, prfp);
+
+	fclose(prfp);
+	fclose(tmpfp);
+
+	print_command_exec(prtmp, cmdline);
+
+	g_free(prtmp);
 }
 
 MsgInfo *procmsg_msginfo_copy(MsgInfo *msginfo)
@@ -1434,8 +1554,13 @@ MsgInfo *procmsg_msginfo_copy(MsgInfo *msginfo)
 
 	MEMBDUP(file_path);
 
-	MEMBDUP(plaintext_file);
-	MEMBCOPY(decryption_failed);
+	if (msginfo->encinfo) {
+		newmsginfo->encinfo = g_new0(MsgEncryptInfo, 1);
+		MEMBDUP(encinfo->plaintext_file);
+		MEMBDUP(encinfo->sigstatus);
+		MEMBDUP(encinfo->sigstatus_full);
+		MEMBCOPY(encinfo->decryption_failed);
+	}
 
 	return newmsginfo;
 }
@@ -1465,8 +1590,17 @@ MsgInfo *procmsg_msginfo_get_full_info(MsgInfo *msginfo)
 
 	full_msginfo->file_path = g_strdup(msginfo->file_path);
 
-	full_msginfo->plaintext_file = g_strdup(msginfo->plaintext_file);
-	full_msginfo->decryption_failed = msginfo->decryption_failed;
+	if (msginfo->encinfo) {
+		full_msginfo->encinfo = g_new0(MsgEncryptInfo, 1);
+		full_msginfo->encinfo->plaintext_file =
+			g_strdup(msginfo->encinfo->plaintext_file);
+		full_msginfo->encinfo->sigstatus =
+			g_strdup(msginfo->encinfo->sigstatus);
+		full_msginfo->encinfo->sigstatus_full =
+			g_strdup(msginfo->encinfo->sigstatus_full);
+		full_msginfo->encinfo->decryption_failed =
+			msginfo->encinfo->decryption_failed;
+	}
 
 	return full_msginfo;
 }
@@ -1510,7 +1644,12 @@ void procmsg_msginfo_free(MsgInfo *msginfo)
 
 	g_free(msginfo->file_path);
 
-	g_free(msginfo->plaintext_file);
+	if (msginfo->encinfo) {
+		g_free(msginfo->encinfo->plaintext_file);
+		g_free(msginfo->encinfo->sigstatus);
+		g_free(msginfo->encinfo->sigstatus_full);
+		g_free(msginfo->encinfo);
+	}
 
 	g_free(msginfo);
 }
