@@ -46,6 +46,7 @@
 #  include <wchar.h>
 #  include <direct.h>
 #  include <io.h>
+#  include <shlobj.h>
 #endif
 
 #include "utils.h"
@@ -467,13 +468,19 @@ gint str_find_format_times(const gchar *haystack, gchar ch)
 gboolean is_next_nonascii(const gchar *s)
 {
 	const gchar *p;
+	gboolean in_quote = FALSE;
 
 	/* skip head space */
 	for (p = s; *p != '\0' && g_ascii_isspace(*p); p++)
 		;
-	for (; *p != '\0' && !g_ascii_isspace(*p); p++) {
-		if (*(guchar *)p > 127 || *(guchar *)p < 32)
+	while (*p != '\0') {
+		if (!in_quote && g_ascii_isspace(*p))
+			break;
+		if (*p == '"')
+			in_quote ^= TRUE;
+		else if (*(guchar *)p > 127 || *(guchar *)p < 32)
 			return TRUE;
+		++p;
 	}
 
 	return FALSE;
@@ -481,12 +488,19 @@ gboolean is_next_nonascii(const gchar *s)
 
 gint get_next_word_len(const gchar *s)
 {
+	const gchar *p = s;
 	gint len = 0;
+	gboolean in_quote = FALSE;
 
-	for (; *s != '\0' && !g_ascii_isspace(*s); s++, len++)
-		;
+	while (*p != '\0') {
+		if (!in_quote && g_ascii_isspace(*p))
+			break;
+		if (*p == '"')
+			in_quote ^= TRUE;
+		++p;
+	}
 
-	return len;
+	return p - s;
 }
 
 /* compare subjects */
@@ -1815,6 +1829,28 @@ const gchar *get_startup_dir(void)
 	return startup_dir;
 }
 
+#ifdef G_OS_WIN32
+static gchar *get_win32_special_folder_path(gint nfolder)
+{
+	gchar *folder = NULL;
+	HRESULT hr;
+
+	if (G_WIN32_HAVE_WIDECHAR_API()) {
+		wchar_t path[MAX_PATH + 1];
+		hr = SHGetFolderPathW(NULL, nfolder, NULL, 0, path);
+		if (hr == S_OK)
+			folder = g_utf16_to_utf8(path, -1, NULL, NULL, NULL);
+	} else {
+		gchar path[MAX_PATH + 1];
+		hr = SHGetFolderPathA(NULL, nfolder, NULL, 0, path);
+		if (hr == S_OK)
+			folder = g_locale_to_utf8(path, -1, NULL, NULL, NULL);
+	}
+
+	return folder;
+}
+#endif
+
 const gchar *get_home_dir(void)
 {
 #ifdef G_OS_WIN32
@@ -1832,19 +1868,38 @@ const gchar *get_home_dir(void)
 #endif
 }
 
+const gchar *get_document_dir(void)
+{
+#ifdef G_OS_WIN32
+	static const gchar *document_dir = NULL;
+	HRESULT hr;
+
+	if (!document_dir) {
+		document_dir = get_win32_special_folder_path(CSIDL_PERSONAL);
+		if (!document_dir)
+			document_dir = get_home_dir();
+	}
+
+	return document_dir;
+#else
+	return get_home_dir();
+#endif
+}
+
 const gchar *get_rc_dir(void)
 {
 	if (!rc_dir) {
 #ifdef G_OS_WIN32
-		const gchar *appdata;
+		gchar *appdata;
 
-		appdata = g_getenv("APPDATA");
+		appdata = get_win32_special_folder_path(CSIDL_APPDATA);
 		if (appdata)
 			rc_dir = g_strconcat(appdata, G_DIR_SEPARATOR_S,
 					     RC_DIR, NULL);
 		else
 			rc_dir = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
 					     RC_DIR, NULL);
+		g_free(appdata);
 #else
 		rc_dir = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
 				     RC_DIR, NULL);
@@ -1948,7 +2003,6 @@ gchar *get_tmp_file(void)
 
 const gchar *get_domain_name(void)
 {
-#ifdef G_OS_UNIX
 	static gchar *domain_name = NULL;
 
 	if (!domain_name) {
@@ -1969,12 +2023,14 @@ const gchar *get_domain_name(void)
 		}
 
 		debug_print("domain name = %s\n", domain_name);
+		if (is_next_nonascii(domain_name)) {
+			g_warning("invalid domain name: %s\n", domain_name);
+			g_free(domain_name);
+			domain_name = "unknown";
+		}
 	}
 
 	return domain_name;
-#else
-	return "unknown";
-#endif
 }
 
 off_t get_file_size(const gchar *file)
@@ -2811,6 +2867,10 @@ FILE *canonicalize_file_stream(FILE *src_fp, gint *length)
 		FILE_OP_ERROR("canonicalize_file_stream", "fgets");
 		err = TRUE;
 	}
+	if (fflush(dest_fp) == EOF) {
+		FILE_OP_ERROR("canonicalize_file_stream", "fflush");
+		err = TRUE;
+	}
 
 	if (err) {
 		fclose(dest_fp);
@@ -2916,6 +2976,45 @@ gchar *normalize_newlines(const gchar *str)
 	return out;
 }
 
+gchar *strchomp_all(const gchar *str)
+{
+	const gchar *p = str;
+	const gchar *newline, *last;
+	gchar *out, *outp;
+
+	out = outp = g_malloc(strlen(str) + 1);
+	while (*p != '\0') {
+		newline = strchr(p, '\n');
+		if (newline) {
+			for (last = newline;
+			     p < last && g_ascii_isspace(*(last - 1)); --last)
+				;
+			strncpy(outp, p, last - p);
+			outp += last - p;
+
+			if (p < newline && *(newline - 1) == '\r') {
+				strncpy(outp, newline - 1, 2);
+				outp += 2;
+			} else {
+				*outp++ = *newline;
+			}
+
+			p = newline + 1;
+		} else {
+			for (last = p + strlen(p);
+			     p < last && g_ascii_isspace(*(last - 1)); --last)
+				;
+			strncpy(outp, p, last - p);
+			outp += last - p;
+			break;
+		}
+	}
+
+	*outp = '\0';
+
+	return out;
+}
+
 FILE *get_outgoing_rfc2822_file(FILE *fp)
 {
 	gchar buf[BUFFSIZE];
@@ -2967,6 +3066,11 @@ FILE *get_outgoing_rfc2822_file(FILE *fp)
 			goto file_error;
 	}
 
+	if (fflush(outfp) == EOF) {
+		FILE_OP_ERROR("get_outgoing_rfc2822_file", "fflush");
+		goto file_error;
+	}
+
 	rewind(outfp);
 	return outfp;
 
@@ -3069,10 +3173,47 @@ gchar *generate_mime_boundary(const gchar *prefix)
 
 gint change_file_mode_rw(FILE *fp, const gchar *file)
 {
-#if HAVE_FCHMOD
-	return fchmod(fileno(fp), S_IRUSR|S_IWUSR);
+#ifdef G_OS_WIN32
+	DWORD attr;
+	BOOL retval;
+
+	if (G_WIN32_HAVE_WIDECHAR_API()) {
+		wchar_t *wpath;
+
+		wpath = g_utf8_to_utf16(file, -1, NULL, NULL, NULL);
+		if (wpath == NULL)
+			return -1;
+
+		attr = GetFileAttributesW(wpath);
+		retval = SetFileAttributesW
+			(wpath, attr & ~(FILE_ATTRIBUTE_READONLY|FILE_ATTRIBUTE_HIDDEN));
+
+		g_free(wpath);
+	} else {
+		gchar *cp_path;
+
+		cp_path = g_locale_from_utf8(file, -1, NULL, NULL, NULL);
+		if (cp_path == NULL)
+			return -1;
+
+		attr = GetFileAttributesA(cp_path);
+		retval = SetFileAttributesA
+			(cp_path, attr & ~(FILE_ATTRIBUTE_READONLY|FILE_ATTRIBUTE_HIDDEN));
+
+		g_free(cp_path);
+	}
+
+	if (retval)
+		return 0;
+	else
+		return -1;
 #else
-	return g_chmod(file, S_IRUSR|S_IWUSR);
+#if HAVE_FCHMOD
+	if (fp)
+		return fchmod(fileno(fp), S_IRUSR|S_IWUSR);
+	else
+#endif
+		return g_chmod(file, S_IRUSR|S_IWUSR);
 #endif
 }
 
@@ -3190,6 +3331,8 @@ FILE *my_tmpfile(void)
 	tmpdir = get_tmp_dir();
 	tmplen = strlen(tmpdir);
 	progname = g_get_prgname();
+	if (!progname)
+		progname = "sylph";
 	proglen = strlen(progname);
 	Xalloca(fname, tmplen + 1 + proglen + sizeof(suffix),
 		return tmpfile());
@@ -3234,6 +3377,11 @@ FILE *str_open_as_stream(const gchar *str)
 		fclose(fp);
 		return NULL;
 	}
+	if (fflush(fp) == EOF) {
+		FILE_OP_ERROR("str_open_as_stream", "fflush");
+		fclose(fp);
+		return NULL;
+	}
 
 	rewind(fp);
 	return fp;
