@@ -197,7 +197,7 @@ struct _AttachInfo
 	gchar *content_type;
 	EncodingType encoding;
 	gchar *name;
-	off_t size;
+	gsize size;
 };
 
 Compose *compose_new		(PrefsAccount	*account,
