@@ -57,6 +57,7 @@
 #include "procmsg.h"
 #include "utils.h"
 #include "gtkutils.h"
+#include "trayicon.h"
 #include "prefs_common.h"
 #include "prefs_account.h"
 #include "prefs_folder_item.h"
@@ -254,6 +255,7 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 					NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/R_ebuild folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
 	{N_("/_Update summary"),	NULL, folderview_update_summary_cb, 0, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Mar_k all read"),		NULL, folderview_mark_all_read_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
@@ -277,6 +279,7 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 					NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/R_ebuild folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
 	{N_("/_Update summary"),	NULL, folderview_update_summary_cb, 0, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Mar_k all read"),		NULL, folderview_mark_all_read_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
@@ -298,6 +301,7 @@ static GtkItemFactoryEntry folderview_news_popup_entries[] =
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/_Update summary"),	NULL, folderview_update_summary_cb, 0, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Mar_k all read"),		NULL, folderview_mark_all_read_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
@@ -2143,6 +2147,8 @@ static void folderview_mark_all_read_cb(FolderView *folderview, guint action,
 	else {
 		procmsg_mark_all_read(item);
 		folderview_update_item(item, FALSE);
+		trayicon_set_tooltip(NULL);
+		trayicon_set_notify(FALSE);
 	}
 }
 
@@ -2498,6 +2504,8 @@ static void folderview_empty_trash_cb(FolderView *folderview, guint action,
 	procmsg_empty_trash(folder->trash);
 	statusbar_pop_all();
 	folderview_update_item(folder->trash, TRUE);
+	trayicon_set_tooltip(NULL);
+	trayicon_set_notify(FALSE);
 
 	open_path = gtk_tree_row_reference_get_path(folderview->opened);
 	if (open_path && sel_path &&
