@@ -625,7 +625,7 @@ void gtkut_tree_view_scroll_to_cell(GtkTreeView *treeview, GtkTreePath *path,
 
 void gtkut_tree_view_fast_clear(GtkTreeView *treeview, GtkTreeStore *store)
 {
-#if GTK_CHECK_VERSION(2, 8, 0)
+#if GTK_CHECK_VERSION(2, 8, 0) && !GTK_CHECK_VERSION(2, 10, 0)
 	gtk_tree_store_clear(store);
 #else
 	/* this is faster than above, but it seems to trigger crashes in
@@ -878,7 +878,12 @@ void gtkut_window_popup(GtkWidget *window)
 	if (new_x != x || new_y != y)
 		gdk_window_move(window->window, new_x, new_y);
 
+	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window), FALSE);
 	gtk_window_present(GTK_WINDOW(window));
+#ifdef G_OS_WIN32
+	/* ensure that the window is displayed at the top */
+	gdk_window_show(window->window);
+#endif
 }
 
 gboolean gtkut_window_modal_exist(void)
