@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwidget.h>
@@ -36,8 +37,8 @@
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkfilesel.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
 
-#include "intl.h"
 #include "main.h"
 #include "inc.h"
 #include "mbox.h"
@@ -55,48 +56,65 @@ static GtkWidget *file_button;
 static GtkWidget *dest_button;
 static GtkWidget *ok_button;
 static GtkWidget *cancel_button;
+static gboolean import_finished;
 static gboolean import_ack;
 
-static void import_create(void);
-static void import_ok_cb(GtkWidget *widget, gpointer data);
-static void import_cancel_cb(GtkWidget *widget, gpointer data);
-static void import_filesel_cb(GtkWidget *widget, gpointer data);
-static void import_destsel_cb(GtkWidget *widget, gpointer data);
-static gint delete_event(GtkWidget *widget, GdkEventAny *event, gpointer data);
-static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);
+static void import_create	(void);
+static void import_ok_cb	(GtkWidget	*widget,
+				 gpointer	 data);
+static void import_cancel_cb	(GtkWidget	*widget,
+				 gpointer	 data);
+static void import_filesel_cb	(GtkWidget	*widget,
+				 gpointer	 data);
+static void import_destsel_cb	(GtkWidget	*widget,
+				 gpointer	 data);
+static gint delete_event	(GtkWidget	*widget,
+				 GdkEventAny	*event,
+				 gpointer	 data);
+static gboolean key_pressed	(GtkWidget	*widget,
+				 GdkEventKey	*event,
+				 gpointer	 data);
 
 gint import_mbox(FolderItem *default_dest)
 {
 	gint ok = 0;
 	gchar *dest_id = NULL;
 
-	if (!window)
-		import_create();
-	else
-		gtk_widget_show(window);
+	import_create();
 
-	gtk_entry_set_text(GTK_ENTRY(file_entry), "");
 	if (default_dest && default_dest->path)
 		dest_id = folder_item_get_identifier(default_dest);
 
 	if (dest_id) {
 		gtk_entry_set_text(GTK_ENTRY(dest_entry), dest_id);
 		g_free(dest_id);
-	} else
-		gtk_entry_set_text(GTK_ENTRY(dest_entry), "");
+	}
 	gtk_widget_grab_focus(file_entry);
 
 	manage_window_set_transient(GTK_WINDOW(window));
 
-	gtk_main();
+	import_finished = FALSE;
+	import_ack = FALSE;
+
+	while (!import_finished)
+		gtk_main_iteration();
 
 	if (import_ack) {
-		gchar *filename, *destdir;
+		const gchar *utf8filename, *destdir;
 		FolderItem *dest;
 
-		filename = gtk_entry_get_text(GTK_ENTRY(file_entry));
+		utf8filename = gtk_entry_get_text(GTK_ENTRY(file_entry));
 		destdir = gtk_entry_get_text(GTK_ENTRY(dest_entry));
-		if (filename && *filename) {
+		if (utf8filename && *utf8filename) {
+			gchar *filename;
+
+			filename = g_filename_from_utf8
+				(utf8filename, -1, NULL, NULL, NULL);
+			if (!filename) {
+				g_warning("faild to convert character set\n");
+				filename = g_strdup(utf8filename);
+			}
+
 			if (!destdir || !*destdir) {
 				dest = folder_find_item_from_path(INBOX_DIR);
 			} else
@@ -110,10 +128,15 @@ gint import_mbox(FolderItem *default_dest)
 				folder_item_scan(dest);
 				folderview_update_item(dest, TRUE);
 			}
+
+			g_free(filename);
 		}
 	}
 
-	gtk_widget_hide(window);
+	gtk_widget_destroy(window);
+	window = NULL;
+	file_entry = dest_entry = NULL;
+	file_button = dest_button = ok_button = cancel_button = NULL;
 
 	return ok;
 }
@@ -128,16 +151,16 @@ static void import_create(void)
 	GtkWidget *dest_label;
 	GtkWidget *confirm_area;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(window), _("Import"));
 	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(delete_event), NULL);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(key_pressed), NULL);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(delete_event), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 
 	vbox = gtk_vbox_new(FALSE, 4);
@@ -156,7 +179,7 @@ static void import_create(void)
 	gtk_container_set_border_width(GTK_CONTAINER(table), 8);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 8);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
-	gtk_widget_set_usize(table, 420, -1);
+	gtk_widget_set_size_request(table, 420, -1);
 
 	file_label = gtk_label_new(_("Importing file:"));
 	gtk_table_attach(GTK_TABLE(table), file_label, 0, 1, 0, 1,
@@ -179,51 +202,60 @@ static void import_create(void)
 	file_button = gtk_button_new_with_label(_(" Select... "));
 	gtk_table_attach(GTK_TABLE(table), file_button, 2, 3, 0, 1,
 			 0, 0, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(file_button), "clicked",
-			   GTK_SIGNAL_FUNC(import_filesel_cb), NULL);
+	g_signal_connect(G_OBJECT(file_button), "clicked",
+			 G_CALLBACK(import_filesel_cb), NULL);
 
 	dest_button = gtk_button_new_with_label(_(" Select... "));
 	gtk_table_attach(GTK_TABLE(table), dest_button, 2, 3, 1, 2,
 			 0, 0, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(dest_button), "clicked",
-			   GTK_SIGNAL_FUNC(import_destsel_cb), NULL);
+	g_signal_connect(G_OBJECT(dest_button), "clicked",
+			 G_CALLBACK(import_destsel_cb), NULL);
 
-	gtkut_button_set_create(&confirm_area,
-				&ok_button,	_("OK"),
-				&cancel_button, _("Cancel"),
-				NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area,
+				      &ok_button, GTK_STOCK_OK,
+				      &cancel_button, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_button);
 
-	gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
-			   GTK_SIGNAL_FUNC(import_ok_cb), NULL);
-	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
-			   GTK_SIGNAL_FUNC(import_cancel_cb), NULL);
+	g_signal_connect(G_OBJECT(ok_button), "clicked",
+			 G_CALLBACK(import_ok_cb), NULL);
+	g_signal_connect(G_OBJECT(cancel_button), "clicked",
+			 G_CALLBACK(import_cancel_cb), NULL);
 
 	gtk_widget_show_all(window);
 }
 
 static void import_ok_cb(GtkWidget *widget, gpointer data)
 {
+	import_finished = TRUE;
 	import_ack = TRUE;
-	if (gtk_main_level() > 1)
-		gtk_main_quit();
 }
 
 static void import_cancel_cb(GtkWidget *widget, gpointer data)
 {
+	import_finished = TRUE;
 	import_ack = FALSE;
-	if (gtk_main_level() > 1)
-		gtk_main_quit();
 }
 
 static void import_filesel_cb(GtkWidget *widget, gpointer data)
 {
 	gchar *filename;
+	gchar *utf8_filename;
 
-	filename = filesel_select_file(_("Select importing file"), NULL);
-	if (filename)
-		gtk_entry_set_text(GTK_ENTRY(file_entry), filename);
+	filename = filesel_select_file(_("Select importing file"), NULL,
+				       GTK_FILE_CHOOSER_ACTION_OPEN);
+	if (!filename) return;
+
+	utf8_filename = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
+	if (!utf8_filename) {
+		g_warning("import_filesel_cb(): faild to convert characer set.");
+		utf8_filename = g_strdup(filename);
+	}
+	gtk_entry_set_text(GTK_ENTRY(file_entry), utf8_filename);
+	g_free(utf8_filename);
+
+	g_free(filename);
 }
 
 static void import_destsel_cb(GtkWidget *widget, gpointer data)
@@ -241,8 +273,9 @@ static gint delete_event(GtkWidget *widget, GdkEventAny *event, gpointer data)
 	return TRUE;
 }
 
-static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
+static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		import_cancel_cb(NULL, NULL);
+	return FALSE;
 }
