@@ -35,11 +35,17 @@
 #include "prefs_common.h"
 #include "utils.h"
 #include "gtkutils.h"
+#include "codeconv.h"
 
 #define TRIM_LINES	25
 
 static LogWindow *logwindow;
 
+static void log_window_print_func	(const gchar	*str);
+static void log_window_message_func	(const gchar	*str);
+static void log_window_warning_func	(const gchar	*str);
+static void log_window_error_func	(const gchar	*str);
+
 static void hide_cb		(GtkWidget	*widget,
 				 LogWindow	*logwin);
 static gboolean key_pressed	(GtkWidget	*widget,
@@ -136,6 +142,9 @@ void log_window_init(LogWindow *logwin)
 	gtk_text_buffer_create_tag(buffer, "error",
 				   "foreground-gdk", &logwindow->error_color,
 				   NULL);
+
+	set_log_ui_func(log_window_print_func, log_window_message_func,
+			log_window_warning_func, log_window_error_func);
 }
 
 void log_window_show(LogWindow *logwin)
@@ -202,8 +211,20 @@ void log_window_append(const gchar *str, LogType type)
 	if (head)
 		gtk_text_buffer_insert_with_tags_by_name
 			(buffer, &iter, head, -1, tag, NULL);
-	gtk_text_buffer_insert_with_tags_by_name
-		(buffer, &iter, str, -1, tag, NULL);
+
+	if (!g_utf8_validate(str, -1, NULL)) {
+		gchar *str_;
+
+		str_ = conv_utf8todisp(str, NULL);
+		if (str_) {
+			gtk_text_buffer_insert_with_tags_by_name
+				(buffer, &iter, str_, -1, tag, NULL);
+			g_free(str_);
+		}
+	} else {
+		gtk_text_buffer_insert_with_tags_by_name
+			(buffer, &iter, str, -1, tag, NULL);
+	}
 
 	if (GTK_WIDGET_VISIBLE(text)) {
 		GtkTextMark *mark;
@@ -214,6 +235,26 @@ void log_window_append(const gchar *str, LogType type)
 	logwindow->lines++;
 }
 
+static void log_window_print_func(const gchar *str)
+{
+	log_window_append(str, LOG_NORMAL);
+}
+
+static void log_window_message_func(const gchar *str)
+{
+	log_window_append(str, LOG_MSG);
+}
+
+static void log_window_warning_func(const gchar *str)
+{
+	log_window_append(str, LOG_WARN);
+}
+
+static void log_window_error_func(const gchar *str)
+{
+	log_window_append(str, LOG_ERROR);
+}
+
 static void hide_cb(GtkWidget *widget, LogWindow *logwin)
 {
 }
