@@ -79,12 +79,14 @@
 #include "gtkutils.h"
 #include "socket.h"
 #include "stock_pixmap.h"
+#include "trayicon.h"
 
 #if USE_GPGME
 #  include "rfc2015.h"
 #endif
 #if USE_SSL
 #  include "ssl.h"
+#  include "sslmanager.h"
 #endif
 
 #ifdef G_OS_WIN32
@@ -166,10 +168,16 @@ int main(int argc, char *argv[])
 	MainWindow *mainwin;
 	FolderView *folderview;
 	GdkPixbuf *icon;
+	GList *iconlist = NULL;
 
 	app_init();
 	parse_cmd_opt(argc, argv);
 
+	sock_init();
+#if USE_SSL
+	ssl_init();
+#endif
+
 	/* check and create (unix domain) socket for remote operation */
 	lock_socket = prohibit_duplicate_launch();
 	if (lock_socket < 0) return 0;
@@ -206,6 +214,9 @@ int main(int argc, char *argv[])
 	set_ui_update_func(gtkut_events_flush);
 	set_progress_func(main_window_progress_show);
 	set_input_query_password_func(input_dialog_query_password);
+#if USE_SSL
+	ssl_set_verify_func(ssl_manager_verify_cert);
+#endif
 
 	CHDIR_EXIT_IF_FAIL(get_home_dir(), 1);
 
@@ -221,8 +232,18 @@ int main(int argc, char *argv[])
 	sock_set_io_timeout(prefs_common.io_timeout_secs);
 
 	gtkut_widget_init();
+
+#ifdef G_OS_WIN32
+	stock_pixbuf_gdk(NULL, STOCK_PIXMAP_SYLPHEED_32, &icon);
+	iconlist = g_list_append(iconlist, icon);
+	stock_pixbuf_gdk(NULL, STOCK_PIXMAP_SYLPHEED_SMALL, &icon);
+	iconlist = g_list_append(iconlist, icon);
+	gtk_window_set_default_icon_list(iconlist);
+	g_list_free(iconlist);
+#else
 	stock_pixbuf_gdk(NULL, STOCK_PIXMAP_SYLPHEED, &icon);
 	gtk_window_set_default_icon(icon);
+#endif
 
 	mainwin = main_window_create
 		(prefs_common.sep_folder | prefs_common.sep_msg << 1);
@@ -508,11 +529,6 @@ static void app_init(void)
 	bind_textdomain_codeset(PACKAGE, CS_UTF_8);
 	textdomain(PACKAGE);
 
-	sock_init();
-#if USE_SSL
-	ssl_init();
-#endif
-
 #ifdef G_OS_UNIX
 	/* ignore SIGPIPE signal for preventing sudden death of program */
 	signal(SIGPIPE, SIG_IGN);
@@ -654,6 +670,8 @@ void app_will_exit(gboolean force)
 			procmsg_remove_all_cached_messages(FOLDER(ac->folder));
 	}
 
+	trayicon_destroy(mainwin->tray_icon);
+
 	/* save all state before exiting */
 	folder_write_list();
 	summary_write_cache(mainwin->summaryview);
@@ -680,7 +698,6 @@ void app_will_exit(gboolean force)
 #if USE_SSL
 	ssl_done();
 #endif
-
 	cleanup_console();
 	sock_cleanup();
 
@@ -1094,6 +1111,14 @@ static void remote_command_exec(void)
 
 	mainwin = main_window_get();
 
+	if (prefs_common.open_inbox_on_startup) {
+		FolderItem *item;
+		item = cur_account && cur_account->inbox
+			? folder_find_item_from_identifier(cur_account->inbox)
+			: folder_get_default_inbox();
+		folderview_select(mainwin->folderview, item);
+	}
+
 	if (cmd.receive_all)
 		inc_all_account_mail(mainwin, FALSE);
 	else if (prefs_common.chk_on_startup)
