@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,6 +71,10 @@
 #include "icons/stock_spam.h"
 #include "icons/stock_hand-signed.h"
 #include "icons/stock_sylpheed.h"
+#include "icons/stock_sylpheed_16.h"
+#include "icons/stock_sylpheed_32.h"
+#include "icons/stock_sylpheed_newmail.h"
+#include "icons/stock_sylpheed_newmail_16.h"
 #include "icons/sylpheed-logo.h"
 
 typedef struct _StockPixmapData	StockPixmapData;
@@ -135,21 +139,13 @@ static StockPixmapData pixmaps[] =
 	{NULL, NULL, NULL, NULL, stock_spam, sizeof(stock_spam), "stock_spam", 24},
 	{NULL, NULL, NULL, NULL, stock_hand_signed, sizeof(stock_hand_signed), "stock_hand-signed", 24},
 	{NULL, NULL, NULL, NULL, stock_sylpheed, sizeof(stock_sylpheed), NULL, 0},
+	{NULL, NULL, NULL, NULL, stock_sylpheed_16, sizeof(stock_sylpheed_16), NULL, 0},
+	{NULL, NULL, NULL, NULL, stock_sylpheed_32, sizeof(stock_sylpheed_32), NULL, 0},
+	{NULL, NULL, NULL, NULL, stock_sylpheed_newmail, sizeof(stock_sylpheed_newmail), NULL, 0},
+	{NULL, NULL, NULL, NULL, stock_sylpheed_newmail_16, sizeof(stock_sylpheed_newmail_16), NULL, 0},
 	{NULL, NULL, NULL, NULL, sylpheed_logo, sizeof(sylpheed_logo), NULL, 0},
 };
 
-/* return newly constructed GtkPixmap from GdkPixmap */
-GtkWidget *stock_pixmap_widget(GtkWidget *window, StockPixmap icon)
-{
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
-
-	g_return_val_if_fail(window != NULL, NULL);
-	g_return_val_if_fail(icon >= 0 && icon < N_STOCK_PIXMAPS, NULL);
-
-	stock_pixmap_gdk(window, icon, &pixmap, &mask);
-	return gtk_pixmap_new(pixmap, mask);
-}
 
 GtkWidget *stock_pixbuf_widget(GtkWidget *window, StockPixmap icon)
 {
@@ -161,6 +157,24 @@ GtkWidget *stock_pixbuf_widget(GtkWidget *window, StockPixmap icon)
 	return gtk_image_new_from_pixbuf(pixbuf);
 }
 
+GtkWidget *stock_pixbuf_widget_scale(GtkWidget *window, StockPixmap icon,
+				     gint width, gint height)
+{
+	GdkPixbuf *pixbuf;
+	GdkPixbuf *scaled_pixbuf;
+	GtkWidget *image;
+
+	g_return_val_if_fail(icon >= 0 && icon < N_STOCK_PIXMAPS, NULL);
+
+	stock_pixbuf_gdk(window, icon, &pixbuf);
+	scaled_pixbuf = gdk_pixbuf_scale_simple(pixbuf, width, height,
+						GDK_INTERP_HYPER);
+	image = gtk_image_new_from_pixbuf(scaled_pixbuf);
+	g_object_unref(scaled_pixbuf);
+
+	return image;
+}
+
 /* create GdkPixmap if it has not created yet */
 gint stock_pixmap_gdk(GtkWidget *window, StockPixmap icon,
 		      GdkPixmap **pixmap, GdkBitmap **mask)
