@@ -89,14 +89,12 @@ static GdkColor emphasis_color = {
 	(gushort)0xcfff
 };
 
-#if 0
 static GdkColor error_color = {
 	(gulong)0,
 	(gushort)0xefff,
 	(gushort)0,
 	(gushort)0
 };
-#endif
 
 #if USE_GPGME
 static GdkColor good_sig_color = {
@@ -147,6 +145,10 @@ static GdkCursor *regular_cursor = NULL;
 static void textview_add_part		(TextView	*textview,
 					 MimeInfo	*mimeinfo,
 					 FILE		*fp);
+#if USE_GPGME
+static void textview_add_sig_part	(TextView	*textview,
+					 MimeInfo	*mimeinfo);
+#endif
 static void textview_add_parts		(TextView	*textview,
 					 MimeInfo	*mimeinfo,
 					 FILE		*fp);
@@ -361,6 +363,9 @@ static void textview_create_tags(TextView *textview)
 	gtk_text_buffer_create_tag(buffer, "emphasis",
 				   "foreground-gdk", &emphasis_color,
 				   NULL);
+	gtk_text_buffer_create_tag(buffer, "error",
+				   "foreground-gdk", &error_color,
+				   NULL);
 #if USE_GPGME
 	gtk_text_buffer_create_tag(buffer, "good-signature",
 				   "foreground-gdk", &good_sig_color,
@@ -432,6 +437,26 @@ void textview_reflect_prefs(TextView *textview)
 					 prefs_common.textview_cursor_visible);
 }
 
+static const gchar *textview_get_src_encoding(TextView *textview,
+					      MimeInfo *mimeinfo)
+{
+	const gchar *charset;
+
+	if (textview->messageview->forced_charset)
+		charset = textview->messageview->forced_charset;
+	else if (!textview->messageview->new_window &&
+		 prefs_common.force_charset)
+		charset = prefs_common.force_charset;
+	else if (mimeinfo->charset)
+		charset = mimeinfo->charset;
+	else if (prefs_common.default_encoding)
+		charset = prefs_common.default_encoding;
+	else
+		charset = NULL;
+
+	return charset;
+}
+
 void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 			   const gchar *file)
 {
@@ -440,7 +465,7 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 	GtkTextMark *mark;
 	GtkTextIter iter;
 	FILE *fp;
-	const gchar *charset = NULL;
+	const gchar *charset;
 	GPtrArray *headers = NULL;
 
 	buffer = gtk_text_view_get_buffer(text);
@@ -450,15 +475,7 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 		return;
 	}
 
-	if (textview->messageview->forced_charset)
-		charset = textview->messageview->forced_charset;
-	else if (prefs_common.force_charset)
-		charset = prefs_common.force_charset;
-	else if (mimeinfo->charset)
-		charset = mimeinfo->charset;
-	else if (prefs_common.default_encoding)
-		charset = prefs_common.default_encoding;
-
+	charset = textview_get_src_encoding(textview, mimeinfo);
 	textview_set_font(textview, charset);
 	textview_clear(textview);
 
@@ -473,6 +490,11 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 	}
 
 	textview_add_parts(textview, mimeinfo, fp);
+#if USE_GPGME
+	if (textview->messageview->msginfo->encinfo &&
+	    textview->messageview->msginfo->encinfo->sigstatus)
+		textview_add_sig_part(textview, NULL);
+#endif
 
 	fclose(fp);
 
@@ -486,7 +508,7 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	gchar buf[BUFFSIZE];
 	const gchar *boundary = NULL;
 	gint boundary_len = 0;
-	const gchar *charset = NULL;
+	const gchar *charset;
 	GPtrArray *headers = NULL;
 	gboolean is_rfc822_part = FALSE;
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
@@ -508,14 +530,7 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		boundary_len = strlen(boundary);
 	}
 
-	if (textview->messageview->forced_charset)
-		charset = textview->messageview->forced_charset;
-	else if (prefs_common.force_charset)
-		charset = prefs_common.force_charset;
-	else if (mimeinfo->charset)
-		charset = mimeinfo->charset;
-	else if (prefs_common.default_encoding)
-		charset = prefs_common.default_encoding;
+	charset = textview_get_src_encoding(textview, mimeinfo);
 
 	if (!boundary && mimeinfo->mime_type == MIME_TEXT) {
 		if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0)
@@ -595,7 +610,7 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	gchar buf[BUFFSIZE];
 	const gchar *boundary = NULL;
 	gint boundary_len = 0;
-	const gchar *charset = NULL;
+	const gchar *charset;
 	GPtrArray *headers = NULL;
 
 	g_return_if_fail(mimeinfo != NULL);
@@ -619,14 +634,7 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	while (fgets(buf, sizeof(buf), fp) != NULL)
 		if (buf[0] == '\r' || buf[0] == '\n') break;
 
-	if (textview->messageview->forced_charset)
-		charset = textview->messageview->forced_charset;
-	else if (prefs_common.force_charset)
-		charset = prefs_common.force_charset;
-	else if (mimeinfo->charset)
-		charset = mimeinfo->charset;
-	else if (prefs_common.default_encoding)
-		charset = prefs_common.default_encoding;
+	charset = textview_get_src_encoding(textview, mimeinfo);
 
 	if (mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
 		headers = textview_scan_header(textview, fp, charset);
@@ -639,11 +647,12 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	}
 
 #if USE_GPGME
-	if (mimeinfo->sigstatus)
-		g_snprintf(buf, sizeof(buf), "\n[%s (%s)]\n",
-			   mimeinfo->content_type, mimeinfo->sigstatus);
-	else
+	if (mimeinfo->parent && mimeinfo->sigstatus) {
+		textview_add_sig_part(textview, mimeinfo);
+		return;
+	}
 #endif
+
 	if (mimeinfo->filename || mimeinfo->name)
 		g_snprintf(buf, sizeof(buf), "\n[%s  %s (%s)]\n",
 			   mimeinfo->filename ? mimeinfo->filename :
@@ -655,22 +664,6 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 			   mimeinfo->content_type,
 			   to_human_readable(mimeinfo->content_size));
 
-#if USE_GPGME
-	if (mimeinfo->sigstatus) {
-		const gchar *color;
-		if (!strcmp(mimeinfo->sigstatus, _("Good signature")))
-			color = "good-signature";
-		else if (!strcmp(mimeinfo->sigstatus, _("Valid signature (untrusted key)")))
-			color = "untrusted-signature";
-		else if (!strcmp(mimeinfo->sigstatus, _("BAD signature")))
-			color = "bad-signature";
-		else
-			color = "nocheck-signature";
-		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, buf, -1,
-							 color, "mimepart",
-							 NULL);
-	} else
-#endif
 	if (mimeinfo->mime_type != MIME_TEXT &&
 	    mimeinfo->mime_type != MIME_TEXT_HTML) {
 		gtk_text_buffer_insert_with_tags_by_name
@@ -739,6 +732,54 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	}
 }
 
+#if USE_GPGME
+static void textview_add_sig_part(TextView *textview, MimeInfo *mimeinfo)
+{
+	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	gchar buf[BUFFSIZE];
+	const gchar *color;
+	const gchar *sigstatus;
+	const gchar *sigstatus_full;
+	const gchar *type;
+
+	if (mimeinfo) {
+		sigstatus = mimeinfo->sigstatus;
+		sigstatus_full = mimeinfo->sigstatus_full;
+		type = mimeinfo->content_type;
+	} else if (textview->messageview->msginfo->encinfo) {
+		sigstatus = textview->messageview->msginfo->encinfo->sigstatus;
+		sigstatus_full =
+			textview->messageview->msginfo->encinfo->sigstatus_full;
+		type = "signature";
+	} else
+		return;
+
+	if (!sigstatus)
+		return;
+
+	g_snprintf(buf, sizeof(buf), "\n[%s (%s)]\n", type, sigstatus);
+
+	if (!strcmp(sigstatus, _("Good signature")))
+		color = "good-signature";
+	else if (!strcmp(sigstatus, _("Valid signature (untrusted key)")))
+		color = "untrusted-signature";
+	else if (!strcmp(sigstatus, _("BAD signature")))
+		color = "bad-signature";
+	else
+		color = "nocheck-signature";
+
+	buffer = gtk_text_view_get_buffer(text);
+	gtk_text_buffer_get_end_iter(buffer, &iter);
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, buf, -1,
+						 color, "mimepart", NULL);
+	if (sigstatus_full)
+		gtk_text_buffer_insert_with_tags_by_name
+			(buffer, &iter, sigstatus_full, -1, "mimepart", NULL);
+}
+#endif
+
 static void textview_add_parts(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 {
 	gint level;
@@ -761,18 +802,22 @@ static void textview_add_parts(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	}
 }
 
-void textview_show_error(TextView *textview)
+static void textview_write_error(TextView *textview, const gchar *msg)
 {
 	GtkTextBuffer *buffer;
 	GtkTextIter iter;
 
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
+	gtk_text_buffer_get_end_iter(buffer, &iter);
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, msg, -1,
+						 "error", NULL);
+}
+
+void textview_show_error(TextView *textview)
+{
 	textview_set_font(textview, NULL);
 	textview_clear(textview);
-
-	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
-	gtk_text_buffer_get_start_iter(buffer, &iter);
-	gtk_text_buffer_insert(buffer, &iter,
-			       _("This message can't be displayed.\n"), -1);
+	textview_write_error(textview, _("This message can't be displayed.\n"));
 }
 
 static void textview_write_body(TextView *textview, MimeInfo *mimeinfo,
@@ -793,6 +838,11 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo,
 			while (fgets(buf, sizeof(buf), tmpfp) != NULL)
 				textview_write_line(textview, buf, conv);
 		fclose(tmpfp);
+	} else {
+		textview_write_error
+			(textview,
+			 _("The body text couldn't be displayed because "
+			   "writing to temporary file failed.\n"));
 	}
 
 	conv_code_converter_destroy(conv);
@@ -1215,6 +1265,13 @@ void textview_clear(TextView *textview)
 
 void textview_destroy(TextView *textview)
 {
+	GtkTextBuffer *buffer;
+	GtkClipboard *clipboard;
+
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
+	clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+	gtk_text_buffer_remove_selection_clipboard(buffer, clipboard);
+
 	textview_uri_list_remove_all(textview->uri_list);
 	textview->uri_list = NULL;
 	g_free(textview);
